/* APP.C
 * For use with GTKeyboard
 * written by David Allen, s2mdalle@titan.vcu.edu
 * http://opop.nols.com/
 * 
 * This file is distributable under the terms of the GNU General Public License
 *
 * define DEBUGGING at compile time for interesting info most people don't
 * want to see.
 */

#define APP_C
#include "master.h"
#include "copying.h"
#include "appdefs.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>

#define V_DEBUGGING 1

#define BUFFER_SIZE 512  /* For opening files */

/********************* MAIN CODE ********************************/

int main(int argc, char *argv[])
{
     /* Check args and do stuff. */
     if(argc>1)
     {
	  if(strcmp(argv[1],"--help")==0    ||
	     strcmp(argv[1],"-h")==0        ||
	     strcmp(argv[1],"-?")==0        ||
	     strcmp(argv[1],"--version")==0 ||
	     strcmp(argv[1],"-v")==0        ||
	     strcmp(argv[1],"-version")==0)
	  {
	       usage(argv[0]);
	  } /* End if */
	  else
	  {
	       init_keyboard_stuff(argv[1]);
	  } /* End else */
     }
     else
     {
	  init_keyboard_stuff((char *)NULL);
     } /* end else */

     /* Initialize the app */
     gtk_init(&argc, &argv);
     /* This is where the real stuff starts happening */

     CreateMainWindow();

     gtk_main();
     return(0);
}

void usage(char *input)
{
     fprintf(stdout,"Usage:  %s filename\n",input);
     fprintf(stdout,"GTKeyboard version %s\n\n",VERSION);
     fprintf(stdout,"Written by David Allen, s2mdalle@titan.vcu.edu\n");
     fprintf(stdout,"http://opop.nols.com\n\n");
     fprintf(stdout,"Released under the GNU General Public License.\n");
     exit(0);
}

void SMACK(GtkWidget *foo, gpointer data)
{    gtk_widget_destroy((GtkWidget *)data); }

void CLONE(void)
{
#ifdef SIGCHLD
	signal(SIGCHLD, SIG_IGN);
#endif /* SIGCHLD */

#ifdef SIGCLD
	signal(SIGCLD, SIG_IGN);
#endif /* SIGCLD */

     if(fork()==0)
     {
	  CreateMainWindow();
     }
}

void init_keyboard_stuff(char *input)
{
     FILE *fp;
     char *home;
     char *filename;

     /* Set up some default values for program parameters */
     options.STATUS_LOGGING=0;
     options.CAPS_LOCK     =0;
     options.SHIFT         =0;
     options.NUMLOCK       =0;
     options.WORD_WRAP      =0;

     options.workingfile=NULL;

     home=getenv("HOME");

     filename = calloc(strlen(home) + strlen("/") + strlen(RC_FILENAME), 
		       sizeof(char));

     strcpy(filename,home);
     strcat(filename,"/");
     strcat(filename,RC_FILENAME);

#ifdef DEBUGGING
     fprintf(stdout,"Reading rcfile %s\n",filename);
#endif /* DEBUGGING */

     fp = fopen(filename,"r");
     if(fp==NULL)
     {
	  fprintf(stdout,"GTKeyboard Warning : ");
	  fprintf(stdout,"Couldn't open %s for reading:  %s\n",filename,
		  strerror(errno));
     } /* End if */
     else
     {
	  /* Parse the rcfile and deal with the different things that it
	   * says to do 
	   */
	  
	  
	  fclose(fp);
     } /* End else */

     if(input!=NULL)
     {
	  options.workingfile=calloc(strlen(input), sizeof(char));
	  strcpy(options.workingfile,input);
     }
     else
     {
	  options.workingfile=(char *)malloc(strlen(STARTING_FILENAME)*sizeof(char));
	  strcpy(options.workingfile, STARTING_FILENAME);
     }

#ifdef DEBUGGING
     fprintf(stderr,"Filename to start with is %s\n",options.workingfile);
#endif
} /* End init_keyboard_stuff */

GtkWidget *widget_from_xpm(GtkWidget *window, gchar **xpm_data)
{
     GdkBitmap *mask;
     GdkPixmap *pixmap_data;
     GtkWidget *pixmap_widget;

     pixmap_data = gdk_pixmap_create_from_xpm_d(window->window, &mask,
						NULL, (gchar **)xpm_data);
     pixmap_widget = gtk_pixmap_new (pixmap_data, mask);
     gtk_widget_show (pixmap_widget);
     return(pixmap_widget);
}

/* New file dialog box */
void NewFile (gpointer callback_data, guint callback_action,
	       GtkWidget *widget)
{  
     GtkWidget *window;
     GtkWidget *okbutton;
     GtkWidget *box;
     GtkWidget *label;
     GtkWidget *entryfield;

     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     
     gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), window);

     gtk_container_border_width(GTK_CONTAINER(window), 10);
     box = gtk_vbox_new(FALSE, 0);

     gtk_container_add(GTK_CONTAINER(window), box);
     label = gtk_label_new("New File");
     gtk_misc_set_alignment(GTK_MISC(label), 0, 0); /* Align the label */
     gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0);
     gtk_widget_show(label);
     okbutton = gtk_button_new_with_label ("OK");
     
     entryfield = gtk_entry_new_with_max_length(40);
     gtk_box_pack_start(GTK_BOX(box), entryfield, FALSE, FALSE, 0);
     gtk_entry_set_text(GTK_ENTRY(entryfield), options.workingfile);
     gtk_widget_show(entryfield);
     gtk_box_pack_start(GTK_BOX(box), okbutton, FALSE, FALSE, 0);
     gtk_widget_show(okbutton);

     gtk_signal_connect(GTK_OBJECT(okbutton), "clicked", 
			GTK_SIGNAL_FUNC(change_filename), entryfield);
     gtk_signal_connect(GTK_OBJECT(window), "destroy_event",
			GTK_SIGNAL_FUNC(smack), window);

     gtk_widget_show_all(window);
}

/* This function changes the name of the file that the program saves to */
void change_filename(GtkWidget *foo, gpointer data)
{
     GtkWidget *grabfrom = (GtkWidget *)data;
     char *changed = "Working filename changed : ";
     char *stuff = gtk_entry_get_text(GTK_ENTRY(grabfrom));

#ifdef DEBUGGING
     fprintf(stderr,"Freeing options.workingfile\n");
     fflush(stderr);
#endif

     if(options.workingfile)
     {
	  g_free(options.workingfile);
	  options.workingfile=NULL;
     }
#ifdef DEBUGGING
     fprintf(stderr,"Done freeing\n");
     fflush(stderr);
#endif     

     options.workingfile = calloc(strlen(stuff), sizeof(char));

#ifdef V_DEBUGGING
     fprintf(stderr,"the new file dialog had %s in it.\n",stuff);
     fflush(stderr);
#endif /* V_DEBUGGING */

     strcpy(options.workingfile, stuff);

     /* Destroy the window - the window's child is the container, the 
      * container's child is the button, (from NewFile) so foo->parent->parent
      * kills the window
      */
     gtk_widget_destroy(foo->parent->parent); /* Hopefully kill the parent */

#ifdef DEBUGGING
     fprintf(stderr,"Filename we're working on is now \"%s\"\n",
	     options.workingfile);
     fflush(stderr);
#endif

     chocolate(changed);
     chocolate(options.workingfile);
     chocolate("\n");
} /* End change_filename */

/* The about dialog within the program */
void About(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
     GtkWidget *aboutwindow;
     GtkWidget *aboutlabel;
     GtkWidget *box;
     GtkWidget *button;
     GtkWidget *foo;
     GtkWidget *vscroll, *table;
     char *version="\nVERSION:  ";
     char *ABOUT="About GTKeyboard menu displayed.\n";
     FILE *fp;
     char buffer[BUFFER_SIZE];
     char *fperror="There was an error opening the information file.\n";

     fp = fopen(ABOUT_INFO, "r");

     aboutwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     aboutlabel  = gtk_text_new(NULL, NULL);

     table = gtk_table_new(2, 2, FALSE);
     gtk_table_attach(GTK_TABLE(table), aboutlabel, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     vscroll = gtk_vscrollbar_new(GTK_TEXT(aboutlabel)->vadj);
     gtk_table_attach(GTK_TABLE(table), vscroll, 1, 2, 0, 1, 
                      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(vscroll); 
     gtk_widget_show(table);
     gtk_widget_show(aboutlabel);

     /* Set dimensions of the window, defined in master.h */
     gtk_widget_set_usize(aboutwindow, ABOUT_WIDTH, ABOUT_HEIGHT);
     gtk_widget_set_usize(aboutlabel,  ABOUT_WIDTH-10, ABOUT_HEIGHT-40);

     gtk_window_set_title(GTK_WINDOW(aboutwindow), "About:  GTKeyboard");
     box         = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(aboutwindow), box);

     foo = gtk_label_new("About this Application...");
     gtk_box_pack_start(GTK_BOX(box), foo, FALSE, FALSE, 0);
     gtk_widget_show(foo);

     gtk_text_set_editable(GTK_TEXT(aboutlabel), FALSE);
     gtk_text_set_point(GTK_TEXT(aboutlabel), 0);

     if(fp!=NULL)
     {
	 while(!feof(fp))
	 {
	      /* More data, Luke! */
	      fgets(buffer,BUFFER_SIZE,fp);
	      
	      /* Insert what we've got */
	      gtk_text_insert(GTK_TEXT(aboutlabel), NULL, NULL, NULL,
			      buffer, strlen(buffer));
	 } /* End while */

	 fclose(fp);  /* Close the file back out */
     } /* End if */
     else
	  gtk_text_insert(GTK_TEXT(aboutlabel), NULL, NULL, NULL,
			  fperror, strlen(fperror));

     gtk_text_insert(GTK_TEXT(aboutlabel), NULL, NULL, NULL,
		     version, strlen(version));
     gtk_text_insert(GTK_TEXT(aboutlabel), NULL, NULL, NULL,
		     VERSION, strlen(VERSION));

     button      = gtk_button_new_with_label("Ok");

     /* Set up ways to get out of this window.  :) */
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(smack), aboutwindow);
     gtk_signal_connect(GTK_OBJECT(aboutwindow), "delete_event",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), aboutwindow);
     
     /* Pack it up, pack it in, let me begin...oh whatever. */
     gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
     gtk_widget_show(aboutlabel);

     gtk_widget_show_all(aboutwindow);

     chocolate(ABOUT);
} /* End About() */

/* LICENSE - displays the license for the HELP menu. */
/* This function is basically an exact copy of the about window with a few
 * modifications for displaying the license 
 */
void LICENSE(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
     GtkWidget *aboutwindow;
     GtkWidget *aboutlabel;
     GtkWidget *box;
     GtkWidget *button;
     GtkWidget *foo;
     GtkWidget *table, *vscroll;
     char *LICENSE = "GTKeyboard license displayed.\n";

     aboutwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     aboutlabel  = gtk_text_new(NULL, NULL);

     table = gtk_table_new(2, 2, FALSE);
     gtk_table_attach(GTK_TABLE(table), aboutlabel, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
                      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     vscroll = gtk_vscrollbar_new(GTK_TEXT(aboutlabel)->vadj);
     gtk_table_attach(GTK_TABLE(table), vscroll, 1, 2, 0, 1, 
                      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(vscroll); 
     gtk_widget_show(table);
     gtk_widget_show(aboutlabel);

     /* Set dimensions of the window, defined in master.h */
     gtk_widget_set_usize(aboutwindow, 570, 545);
     gtk_widget_set_usize(aboutlabel, 560, 505);

     gtk_window_set_title(GTK_WINDOW(aboutwindow), "License:");

     box         = gtk_vbox_new(FALSE, 0);

     gtk_container_add(GTK_CONTAINER(aboutwindow), box);

     foo = gtk_label_new("GTKeyboard's License");
     gtk_box_pack_start(GTK_BOX(box), foo, FALSE, FALSE, 0);
     gtk_widget_show(foo);

     gtk_text_set_editable(GTK_TEXT(aboutlabel), FALSE);
     gtk_text_set_point(GTK_TEXT(aboutlabel), 0);
     gtk_text_insert(GTK_TEXT(aboutlabel), NULL, NULL, NULL,
		     license, strlen(license));

     button      = gtk_button_new_with_label("Ok");

     /* Set up ways to get out of this window.  :) */
     gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(smack), aboutwindow);
     gtk_signal_connect(GTK_OBJECT(aboutwindow), "delete_event",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), aboutwindow);

     /* Pack it up, pack it in, let me begin...oh whatever. */
     gtk_box_pack_start(GTK_BOX(box), table, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);
     gtk_widget_show(button);
     gtk_widget_show(aboutlabel);

     gtk_widget_show_all(aboutwindow);

     chocolate(LICENSE);
} /* End LICENSE */

/**
 * Set up the file dialog and do all that funny stuff relating to returning
 * the name of the file they want, etc.
 */

void OpenFile (gpointer callback_data, guint callback_action,
	      GtkWidget *widget)
{
     if(selectedfile)
     {
	  free(selectedfile); /* Empty it out */
	  selectedfile = NULL;
     }

     /* Set up file selection dialog */
     fileselection = gtk_file_selection_new(FILE_SELECTION_TITLE);
     
     /* Hook up destruction of widget */
     gtk_signal_connect(GTK_OBJECT(fileselection), "destroy",
			(GtkSignalFunc) DestroyDialog, &fileselection);

     /* Hook up the action of clicking the OK button */
     gtk_signal_connect(GTK_OBJECT(
	  GTK_FILE_SELECTION(fileselection)->ok_button),
			"clicked", (GtkSignalFunc)file_ok_button, 
			fileselection);

     /* Did the bastard click the cancel button? */
     gtk_signal_connect(GTK_OBJECT(
	  GTK_FILE_SELECTION(fileselection)->cancel_button),
			"clicked", (GtkSignalFunc)file_cancel_button,
			fileselection);

     /* Set up the filename selection dialog */
     gtk_file_selection_set_filename (GTK_FILE_SELECTION(fileselection), "");
     gtk_widget_show(fileselection);

     /* Keep focus */
     gtk_grab_add(fileselection);

     /* Get it all kranking */
     gtk_main();

} /* End newfile */

int DestroyDialog (GtkWidget *widget, gpointer *data)
{
#ifdef DEBUGGING
     g_message("Widget Closed\n");
#endif 
     gtk_grab_remove (widget);
     gtk_main_quit ();
     return (FALSE);
}

/* This function nukes EVERYTHING that is in the text window passed to it
 * via data
 */

void clear_all_text(GtkWidget *w, gpointer data)
{
     GtkWidget *window;
     GtkWidget *yes;
     GtkWidget *no;
     GtkWidget *container;
     GtkWidget *subcontainer;
     GtkWidget *label;

     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     subcontainer = gtk_hbox_new(FALSE,0);
     container = gtk_vbox_new(FALSE, 0);
     
     /* Make the buttons nice and WIDE */
     yes = gtk_button_new_with_label("           Yes           ");
     no =  gtk_button_new_with_label("           NO!           ");
     label = gtk_label_new("Are you sure you want to clear ALL text?");

     gtk_widget_show(yes);
     gtk_widget_show(no);
     gtk_widget_show(label);

     /* connect signal to Really go do it. */
     gtk_signal_connect(GTK_OBJECT(yes), "clicked",
			GTK_SIGNAL_FUNC(clear_all_text_really), data);

     /* Just destroy this window if the answer is no */
     gtk_signal_connect(GTK_OBJECT(no), "clicked",
			GTK_SIGNAL_FUNC(smack), window);

     gtk_box_pack_start(GTK_BOX(subcontainer), yes, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(subcontainer), no, FALSE, FALSE, 0);
     gtk_widget_show(subcontainer);

     gtk_box_pack_start(GTK_BOX(container), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), subcontainer, FALSE, FALSE, 0);
     gtk_widget_show(container);

     /* Add the whole damn thing into the window */
     gtk_container_add(GTK_CONTAINER(window), container);

     gtk_widget_show_all(window);

} /* End clear_all_text */

/* For those that are sure that they want to delete everything. */
void clear_all_text_really(GtkWidget *widget, gpointer data)
{
     GtkWidget *foobar = (GtkWidget *)data;
     int length = gtk_text_get_length(GTK_TEXT((GtkWidget *)data));
     
     gtk_text_set_point(GTK_TEXT(foobar), length);
     
     /* Now delete backward ALL characters. */
     gtk_text_backward_delete(GTK_TEXT(foobar), (guint)length);
     
     /* Reset point to be 0 */
     gtk_text_set_point(GTK_TEXT(foobar), 0);
     /* The yes button's parent is a hbox, then vbox, then window so 3x
      * parent
      */
     gtk_widget_destroy(widget->parent->parent->parent);
}

/* Used by init_keyboard_stuff if needed */
int load_file(char *filename, GtkWidget *textbox)
{
     char buffer[BUFFER_SIZE];
     FILE *fp;
     
     fp = fopen(filename,"r");

     if(fp==NULL)
     {
	  fprintf(stderr,"Open of %s failed!  : %s\n",filename,
		  strerror(errno));
	  return(1);
     }
     else
     {
	  gtk_text_freeze(GTK_TEXT(textbox));
	  while(!feof(fp))
	  {
	       fgets(buffer,BUFFER_SIZE,fp);
	       gtk_text_insert(GTK_TEXT(textbox), NULL, NULL, NULL,
			       buffer,strlen(buffer));
	  }
	  gtk_text_thaw(GTK_TEXT(textbox));
	  fclose(fp);
     } /* End else */
     return(0);
} /* End load_file */

/* What happens when you click the ok button within the open file selection
 * dialog
 */
void file_ok_button (GtkWidget *w, GtkFileSelection *fs)
{
    char *tempfile;
    char buffer[BUFFER_SIZE]; 
    int length;
    FILE *fp;

    /* --- Get the name --- */
    tempfile = gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));

    /* Do not repaint until we're done */
    gtk_text_freeze(GTK_TEXT(output_text));
    
#ifdef DEBUGGING
    g_message("Getting %s\n", tempfile);
    fprintf(stderr,"Freeing options.workingfile if needed...\n");
    fflush(stderr);
#endif

    /* Free the old file information pointer */
    if(options.workingfile)
    {
	 g_free(options.workingfile);
	 options.workingfile=NULL;
    }

    /* --- Allocate space and save it. --- */
    options.workingfile = calloc(strlen(tempfile), sizeof(char));
    strcpy (options.workingfile, tempfile);

    /* Now clear the old text widget, load the file and put it's contents
     * into the window output_text
     */
    
    length = gtk_text_get_length(GTK_TEXT(output_text));

    /* Set the point to the last character */
    gtk_text_set_point(GTK_TEXT(output_text), length);

    /* Now delete backward ALL characters. */
    gtk_text_backward_delete(GTK_TEXT(output_text), (guint)length);

    /* Reset point to be 0 */
    gtk_text_set_point(GTK_TEXT(output_text), 0);

    /* Fresh widget, nothing in it.  */
    if((fp=fopen(options.workingfile,"r"))!=NULL)
    {
	 /* While there is still data left to read, write it into the text
	  * widget window 
	  */
	 
	 while(!feof(fp))
	 {
#ifdef DEBUGGING	      
	      fprintf(stderr,"Reading info...\n");
	      fflush(stderr);
#endif
	      /* More data, Luke! */
	      fgets(buffer,BUFFER_SIZE,fp);

#ifdef DEBUGGING
	      fprintf(stderr,"Buffer=\"%s\"",buffer);
#endif
	      /* Insert what we've got */
	      gtk_text_insert(GTK_TEXT(output_text), NULL, NULL, NULL,
			      buffer, strlen(buffer));
	 }

	 /* Close the file */
	 fclose(fp);
	 /* And now reset the point to 0 */
	 gtk_text_set_point(GTK_TEXT(output_text), 0);
    } /* End if fp!=NULL */
    else
    {
	 fprintf(stderr,"Open of %s failed: %s\n",options.workingfile,strerror(errno));
    }

    /* --- Destroy the file selection --- */
    gtk_widget_destroy (fileselection);

    gtk_text_thaw(GTK_TEXT(output_text));
} /* End file_ok_button */

/* this function creates a yes no popup menu, connects the yes button to 
 * the gtksignalfunc yes, the no button to gtksignalfunc no, with their
 * data pointers being yesdata and nodata - sets the title of the window
 * and a label at the top of the window to be labelname, sets up the 
 * necessary crap, so you can get on with life 
 */

void yes_no_popup_dialog(char *labelname, GtkSignalFunc yes, GtkSignalFunc no,
			 gpointer yesdata, gpointer nodata)
{
     GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
     GtkWidget *label  = gtk_label_new (labelname);
     GtkWidget *yesb   = gtk_button_new_with_label("   Yes   ");
     GtkWidget *nob    = gtk_button_new_with_label("   No    ");
     GtkWidget *box    = gtk_vbox_new(FALSE, 0);
     GtkWidget *cont   = gtk_hbox_new(FALSE, 0);
     
     gtk_box_pack_start(GTK_BOX(box), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(cont), yesb, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(cont), nob,  FALSE, FALSE, 0);
     
     gtk_box_pack_start(GTK_BOX(box), cont, FALSE, FALSE, 0);
     gtk_widget_show(label);
     gtk_widget_show(yesb);
     gtk_widget_show(nob);
     gtk_widget_show(box);
     gtk_container_add(GTK_CONTAINER(window), box);

     /* If yesdata and nodata aren't supplied, then simply destroy the
      * popup when they are clicked 
      */
     if(!yesdata)
	  gtk_signal_connect(GTK_OBJECT(yesb), "clicked",
			     GTK_SIGNAL_FUNC(SMACK), window);
     else
	  gtk_signal_connect(GTK_OBJECT(yesb), "clicked",
			     GTK_SIGNAL_FUNC(yes), yesdata);

     if(!nodata)
	  gtk_signal_connect(GTK_OBJECT(nob), "clicked",
			     GTK_SIGNAL_FUNC(SMACK), window);
     else
	  gtk_signal_connect(GTK_OBJECT(nob),  "clicked",
			     GTK_SIGNAL_FUNC(no), nodata);

     /* Connect regular signals to the window */
     gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(gtk_widget_destroy), window);
     
     gtk_window_set_title(GTK_WINDOW(window), labelname);
     gtk_widget_show(window);

     gtk_widget_show_all(window);
} /* End yes/no popup dialog */


/* This is called in the menus to change the output */
void change_output(GtkWidget *w, int data)
{
     int foo = data;
     char *def = "/dev/null";
     char *changed = "\nOutput file changed to : ";
     
     if(options.workingfile)
     {
	  g_free(options.workingfile);
	  options.workingfile=NULL;
     } /* End if */

     switch(foo)
     {
     case OUTPUT_CONSOLE:
	  options.workingfile = calloc(strlen(OUTPUT_DEVICE_CONSOLE), 
				       sizeof(char));
	  strcpy(options.workingfile, OUTPUT_DEVICE_CONSOLE); 
	  chocolate(changed);
	  chocolate(options.workingfile);
	  chocolate("\n");
	  break;
     case OUTPUT_PRINTER:
	  options.workingfile = calloc(strlen(OUTPUT_DEVICE_PRINTER), 
				       sizeof(char));
	  strcpy(options.workingfile, OUTPUT_DEVICE_PRINTER); 
	  chocolate(changed);
	  chocolate(options.workingfile);
	  chocolate("\n");
	  break;
     case OUTPUT_TEMPFILE:
	  options.workingfile = calloc(strlen(OUTPUT_DEVICE_TEMPFILE), 
				       sizeof(char));
	  strcpy(options.workingfile, OUTPUT_DEVICE_TEMPFILE);
	  chocolate(changed);
	  chocolate(options.workingfile);
	  chocolate("\n");
	  break;
     default:
	  options.workingfile = calloc(strlen(def), sizeof(char));
	  strcpy(options.workingfile, def); 
	  chocolate(changed);
	  chocolate(options.workingfile);
	  chocolate("\n");
	  break;
     } /* End switch */
} /* End void change_output */






