/* CALLBACKS.C 
 * This file is for use with GTKeyboard written by David Allen
 * s2mdalle@titan.vcu.edu
 * http://opop.nols.com/
 * 
 * This file is released under the terms of the GNU GPL.  Please see 
 * http://www.gnu.org for more details.
 */
#define CALLBACKS_C 1
#define APP_C 1   /* So this file can get at app.c's data in master.h */
#include "master.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>


/* Temp debugging spits out some really annoying messages that are pretty
 * much just for me and whoever else hacks, so leave it commented unless you
 * want the program to spew ugly shit all over the place.  Same with DEBUGGING
 * and CONSOLE_OUTPUT
 */
/* #define TEMP_DEBUGGING
 * #define DEBUGGING
 * #define CONSOLE_OUTPUT
 */

/* get_current_dir_name requires __USE_GNU defined - just in case other
 * systems don't do it
 */
#define __USE_GNU      
#define MESSAGE_WIDTH  200
#define MESSAGE_HEIGHT 100

void print_key(GtkWidget *widget, char *data)
{
     char buffer[2]="A\0";
     char *toinsert = &buffer[0];
     static char *nobkspc="GTKeyboard:  Can't backspace.\n";
     char toss;

#ifdef TEMP_DEBUGGING
     fprintf(stdout,"Print_key called with \"%s\"\n",data);
#endif

     if(options.CAPS_LOCK==1)
     {
	  if(isalpha(*data))
	       toss=*data++;

	  sprintf(buffer,"%c",*data);
     }
     else if(options.SHIFT==1)
     {
	  toss=*data++;
	  sprintf(buffer,"%c",*data);
	  /* Toggle shift status - shift_on ignores the widget arg so just
	   * give it a null */
	  options.SHIFT=0;
     }
     else
     {
	  sprintf(buffer,"%c",*data);
     }
     
     if(*data=='\b')   /* High and low are the same */
     {
#ifdef TEMP_DEBUGGING
	  fprintf(stdout,"Inserting backspace\n");
#endif	  
	  if(gtk_text_backward_delete(GTK_TEXT(output_text), (guint)1)==FALSE)
	  {
	       chocolate(nobkspc);
#ifdef DEBUGGING
	       fprintf(stderr,"GTKeyboard: Can't BACKSPACE in text widget\n");
#endif /* DEBUGGING */

	  } /* end if */
     } /* End if */
     else
     {
	  FREEZE(output_text);
	  gtk_text_insert(GTK_TEXT(output_text), NULL, NULL, NULL, 
			  toinsert, strlen(toinsert));
	  THAW(output_text);
     }

#ifdef CONSOLE_OUTPUT
     fprintf(stderr,"%s",buffer);
     fflush(stderr);
#endif /* CONSOLE_OUTPUT */

} /* END PRINT_KEY */

/* Copy, paste, cut functions */

void CutText(GtkWidget *w, gpointer data)
{
     gtk_editable_cut_clipboard(GTK_EDITABLE(output_text));
}

void CopyText(GtkWidget *w, gpointer data)
{
     gtk_editable_copy_clipboard(GTK_EDITABLE(output_text));
}

void PasteText(GtkWidget *w, gpointer data)
{
     gtk_editable_paste_clipboard(GTK_EDITABLE(output_text));
}

/* Status report just prints out what options.CAPS_LOCK and options.SHIFT are to the status
 * widget
 */
void status_report(GtkWidget *widget, gpointer data)
{
     char *temp = calloc(200, sizeof(char));
     char *on="ON ", *off="OFF ";
     char *wd = calloc(125,sizeof(char));

     getcwd(wd, sizeof(wd));

     sprintf(temp,"--== STATUS ==--\n");
     strcat(temp,"SHIFT is ");
     if(options.SHIFT==1)
	  strcat(temp,on);
     else
	  strcat(temp,off);
     strcat(temp,"\nCAPS_LOCK is ");
     if(options.CAPS_LOCK==1)
	  strcat(temp, on);
     else
	  strcat(temp, off);

     strcat(temp,"\nSTATUS LOGGING is ");
     if(options.STATUS_LOGGING==1)
	  strcat(temp,on);
     else
	  strcat(temp,off);
     strcat(temp,"\nWORD WRAP is ");

     if(options.WORD_WRAP)
	  strcat(temp,on);
     else
	  strcat(temp,off);

     strcat(temp,"\nNUM LOCK is ");
     if(options.NUMLOCK)
	  strcat(temp,on);
     else
	  strcat(temp,off);
     strcat(temp,"\nOur workingfile is ");
     strcat(temp,options.workingfile);

     strcat(temp,"\n");

     gtk_text_insert(GTK_TEXT((GtkWidget *)data), NULL, NULL, NULL, 
		     temp, strlen(temp));

} /* End status report */


int delete_event(GtkWidget *foo, gpointer data)
{
     g_message("Quitting...\n");

     return(FALSE);
} /* End delete_event */

void QuitApp(gpointer callback_data, guint callback_action,
	     GtkWidget *widget)
{
#ifdef DEBUGGING
     g_message("ItemFactory:  activated \"%s\"",
               gtk_item_factory_path_from_widget(widget));
#endif
     CloseApp((GtkWidget *)NULL, NULL);

}

void smack(GtkWidget *w, gpointer data)
{
     /* Get it out of our faces */
     gtk_widget_destroy((GtkWidget *)data);
}

void save_output_text(GtkWidget *w, gpointer data)
{
     char *buffer;
     FILE *fp;
     int nchars;
     GtkWidget *foobar = (GtkWidget *)data;

     if(!foobar)        /* Sometimes the menu calls this and sometimes it's
			 * a button, so compensate for both and make up the
			 * args if they're not there.  :)
			 */
     {
	  foobar=output_text;
     }

#ifdef DEBUGGING
     fprintf(stderr,"PRE-ERROR:  %s\n",strerror(errno));
     fprintf(stderr,"SAVING:  \"%s\"\n",options.workingfile);
#endif

     if((fp=fopen(options.workingfile, "a"))!=NULL)
     {
	  /* Freeze so user can't edit while we're saving */
	  gtk_text_freeze(GTK_TEXT(foobar));
	  
	  buffer = gtk_editable_get_chars (
	       GTK_EDITABLE (foobar),
	       (gint) 0,
	       (gint) gtk_text_get_length (GTK_TEXT (foobar)));
	  
	  nchars = fwrite (buffer, sizeof (char), strlen (buffer), fp);

	  if(nchars!=strlen(buffer))
	  {
	       save_output_text_error("Couldn't write enough bytes!");
	  }
	  g_free(buffer);
	  
	  /* Unfreeze so editing can keep going */
	  gtk_text_thaw(GTK_TEXT(foobar));
	  fflush(fp);
	  fclose(fp);
	  save_output_text_successful(options.workingfile);
     }
     else
     {
#ifdef DEBUGGING
	  fprintf(stderr,"ERROR:  %s\n",strerror(errno));
#endif
	  /* Put up a popup with the errno error in it. */
	  save_output_text_error(strerror(errno));
     }
} /* End save_output_text */

void save_output_text_error(char *error)
{
     GtkWidget *window;
     GtkWidget *button;
     GtkWidget *label;
     GtkWidget *label2;
     GtkWidget *label3;
     GtkWidget *label4;
     GtkWidget *container;

     window = gtk_window_new(GTK_TOPLEVEL);
     container = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(container);

     gtk_widget_set_usize(window, MESSAGE_WIDTH, MESSAGE_HEIGHT);

     label  = gtk_label_new("File save error:");
     label2 = gtk_label_new(error);
     label3 = gtk_label_new(options.workingfile);
     label4 = gtk_label_new("Oops!");

     button = gtk_button_new_with_label("Close");

     gtk_signal_connect(GTK_OBJECT(button), "clicked", 
			GTK_SIGNAL_FUNC(smack), window);
     gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(smack), window);
     
     gtk_box_pack_start(GTK_BOX(container), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), label2, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), label3, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), label4, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), button, FALSE, FALSE, 0);

     gtk_widget_show(label);
     gtk_widget_show(label2);
     gtk_widget_show(label3);
     gtk_widget_show(label4);
     gtk_widget_show(button);
     
     gtk_container_add(GTK_CONTAINER(window), container);
     
     gtk_widget_show_all(window);
} /* End save_output_text_error */

/* Save_output_text_successful - happens when we saved a file */

void save_output_text_successful(char *filename)
{
     GtkWidget *window;
     GtkWidget *button;
     GtkWidget *label;
     GtkWidget *label2;
     GtkWidget *label3;
     GtkWidget *container;

     window = gtk_window_new(GTK_TOPLEVEL);
     container = gtk_vbox_new(FALSE, 0);
     gtk_widget_show(container);

     gtk_widget_set_usize(window, MESSAGE_WIDTH, MESSAGE_HEIGHT);

     label  = gtk_label_new("Data successfully written to:");
     label2 = gtk_label_new(filename);
     label3 = gtk_label_new(options.workingfile);
     button = gtk_button_new_with_label("Close");

     gtk_signal_connect(GTK_OBJECT(button), "clicked", 
			GTK_SIGNAL_FUNC(smack), window);
     gtk_signal_connect(GTK_OBJECT(window), "destroy_event",
			GTK_SIGNAL_FUNC(smack), window);
     
     gtk_box_pack_start(GTK_BOX(container), label, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), label2, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), label3, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(container), button, FALSE, FALSE, 0);

     gtk_widget_show(label);
     gtk_widget_show(label2);
     gtk_widget_show(label3);
     gtk_widget_show(button);
     
     gtk_container_add(GTK_CONTAINER(window), container);
     gtk_widget_show_all(window);
}


/* Showmenu */
void Showmenu(gpointer callback_data, guint callback_action,
	      GtkWidget *widget)
{
#ifdef DEBUGGING
     g_message("ItemFactory: activated \"%s\", action %d",
               gtk_item_factory_path_from_widget(widget),
               (int) callback_action);
#endif
} /* End showmenu */

/* Turn status logging on or off - only saves at exit */
void status_logging_toggle(void)
{
     char *on = "Status logging is ON\n";
     char *off = "Status logging is OFF\n";

     if(options.STATUS_LOGGING==1)
     {
	  options.STATUS_LOGGING=0;
	  chocolate(off);
     } /* End if */
     else 
     {
	  options.STATUS_LOGGING=1;
	  chocolate(on);
     } /* End else */
} /* ENd status_logging_toggle */

gint CloseApp (GtkWidget *widget, gpointer data)
{
     /* Print a carriage return just to pretty things up so to speak */
     char *filename = "gtkeyboard.status.txt";
     char *datafoo;
     FILE *fp;

     if(options.STATUS_LOGGING)
     {
	  fp = fopen(filename,"w");
	  if(fp==NULL)
	  {
	       fprintf(stderr,"Couldn't open %s for writing : %s\n",filename,
		       strerror(errno));
	       fprintf(stderr,"Status log NOT written.\n");
	  }
	  else
	  {
	       datafoo = gtk_editable_get_chars(GTK_EDITABLE(status), 
						(gint) 0, (gint) -1);
	       fprintf(fp,"%s",datafoo);
	       fflush(fp);
	       fclose(fp);
	       printf("Status log successfully written to %s\n",filename);
	  } /* End else */
     } /* End if options.STATUS_LOGGING */
     else
     {
	  fprintf(stdout,"Status logging wasn't enabled.\n");
	  fprintf(stdout,"Status log NOT written.\n");
     } /* End else */

     fflush(stderr);
     fflush(stdout);
     /* Now actually quit */
     gtk_main_quit();
     return(TRUE);
} /* End closeapp */

/* Function exits the program and dies */
int byebye(void)
{
     gtk_main_quit();
     return(TRUE);
} /* End byebye */

void file_cancel_button(GtkWidget *w, GtkFileSelection *fs)
{
     /* Kill the file selection dialog */
     gtk_widget_destroy(fileselection);
}

void display_key(GtkWidget *widget, gpointer data)
{
     gtk_text_insert(GTK_TEXT(output_text), NULL, NULL, NULL,
		     (char *)data, strlen((char *)data));
}

void capslock_toggle(GtkWidget *w, gpointer data)
{
     GtkWidget *stat = (GtkWidget *)data;

     /* Whatever it currently is, swtich it */
     if(options.CAPS_LOCK==0)
     {	  
	  options.CAPS_LOCK=1;
	  gtk_text_insert(GTK_TEXT(stat), NULL, NULL, NULL,
			  CAPS_LOCK_IS_ON, strlen(CAPS_LOCK_IS_ON));
     }
     else
     {
	  options.CAPS_LOCK=0;
	  gtk_text_insert(GTK_TEXT(stat), NULL, NULL, NULL,
			  CAPS_LOCK_IS_OFF, strlen(CAPS_LOCK_IS_OFF));
     }

#ifdef DEBUGGING
     fprintf(stderr,"CAPS_LOCK is now = %d\n",options.CAPS_LOCK);
#endif DEBUGGING

} /* End capslock_toggle */

void shift_on(GtkWidget *w, gpointer data)
{
     GtkWidget *stat = (GtkWidget *)data;

     /* Turn shift on */
     options.SHIFT=1;

     gtk_text_insert(GTK_TEXT(stat), NULL, NULL, NULL,
                     SHIFT_IS_ON, strlen(SHIFT_IS_ON));
     
#ifdef DEBUGGING
     fprintf(stderr,"Shift is currently %d\n",options.SHIFT);
#endif
} /* End shift_on */



