/* MASTER.H
 * For use with GTKeyboard
 * Written by David Allen
 * s2mdalle@titan.vcu.edu http://opop.nols.com
 *
 * This file is distributable under the terms of the GNU General Public License
 *
 * This file contains all the prototypes, static variable declarations, ADTs,
 * and all that crap.  It's kinda smooshed together into one file, but hey,
 * COPE.  :)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>
#include <assert.h>
#include "appdefs.h"
//#include "cursor_foo.h"
#include <time.h>
#include <sys/time.h>

/* Take the following define out if you don't want to compile with HTML menus
 */
/* #define INCLUDE_HTML              1 */

#define ABOUT_INFO                "/usr/local/bin/.gtkeyboard.about.data"
#define RC_FILENAME               ".gtkeyboardrc"
#define STARTING_FILENAME         "/tmp/GTKeyboard"
#define APPNAME                   "GTKeyboard, written by David Allen"
#define WIDTH                     550
#define HEIGHT                    600
#define ABOUT_WIDTH               450
#define ABOUT_HEIGHT              350
#define FILE_SELECTION_TITLE      "Open a file"
#define CELL_WIDTH                40
#define CELL_HEIGHT               40
#define GRID_ROWS                 10
#define GRID_COLS                 10
#define VERSION                   "0.7"
#define SHIFT_IS_ON               "The shift key is now ON\n"
#define SHIFT_IS_OFF              "The shift key is now OFF\n"
#define CAPS_LOCK_IS_ON           "The Caps Lock key is now ON\n"
#define CAPS_LOCK_IS_OFF          "The Caps Lock key is now OFF\n"
#define STATUS_WIDTH              350
#define STATUS_HEIGHT             125
#define OUTPUT_HEIGHT             250
#define OUTPUT_WIDTH              350
#define STATUS_INTRO              "This is the status buffer.\nHere you will see messages relating to what's going on with the keyboard.  It's useful for some people.  It will display non-output relevant messages.\n\n"

/* Quick function macros */
#define GOTO_EOF                  gtk_text_set_point(GTK_TEXT(output_text), 0)
#define GOTO_BOF                  gtk_text_set_point(GTK_TEXT(output_text), \
                                  gtk_text_get_length(GTK_TEXT(output_text)))
#define CLEAR_ALL                 gtk_editable_delete_text(\
                                  GTK_EDITABLE(output_text), 0, -1)
#define FREEZE(foo)               gtk_text_freeze(GTK_TEXT(foo))
#define THAW(foo)                 gtk_text_thaw(GTK_TEXT(foo))

/* HTML defines */

/* Header defines for html_header(); */
#define H1                       1
#define H2                       2
#define H3                       3
#define H4                       4
#define H5                       5
#define H6                       6

/*###################################################################*/

/* ADTs */
typedef struct {    /* Not currently used but keeping it around anyway for 
		     * now */
     char low;
     char high;
} keypress;

/* Program configuration settings */
typedef struct
{
     int STATUS_LOGGING;              /* Toggle */
     int CAPS_LOCK;                   /* Toggle */
     int SHIFT;                       /* Toggle */
     int WORD_WRAP;                   /* Toggle */
     int NUMLOCK;                     /* Toggle */
     int misc;                        /* As of yet unused */
     char *workingfile;               /* Reallocated as needed */
} GTKeyboardOptions;

/*###################################################################*/
/******************* PROTOTYPES *******************************/
/* By file */

/* Truly_misc.c */
GtkWidget *xpm_to_widget                     (GtkWidget *foo, 
					      gchar **xpm_data);
void splash                                  (void);
void not_done_yet                            (void);

/* for shortcuts.c */
void SC_quote                                (void);
void SC_single_quote                         (void);
void SC_backquote                            (void);
void SC_book_quote                           (void);
void SC_pseudo_underline                     (void);
void SC_angle                                (void);
void SC_double_angle                         (void);
void SC_emphasize                            (void);
void SC_surround                             (void);
void SC_parens                               (void);
void SC_smiley                               (void);
void SC_winky                                (void);
void SC_hihat                                (void);
void SC_rose                                 (void);
void SC_paragraph                            (void);
void SC_timestamp                               (void);

/* For build_keyboard.c */
GtkWidget *build_keyboard                    (GtkWidget *input);
GtkWidget *build_fkeys                       (void);
GtkWidget *makekeybutton                     (char input, int subscript, 
					      GtkWidget *outwrite, char foo[]);
GtkWidget *makekeypadkey                     (char input, int subscript, 
					      GtkWidget *outwrite, char foo[]);
GtkWidget *build_keypad                      (GtkWidget *input);
GtkWidget *build_cursorpad                   (GtkWidget *foo);

/* For misc_cursor.c */
void cursor_left                             (void);
void cursor_right                            (void);
void cursor_up                               (void);
void setup_vals                              (void);
void cursor_right_x_times                    (size_t input);
void cursor_left_x_times                     (size_t input);
void update_cursor                           (void);
void freezeit                                (void);
void thawit                                  (void);
void toggle_wordwrap                         (void);
void toggle_numlock                          (void);
void delete_key                              (GtkWidget *w, gpointer data);

/* Goto beginning/end of file/line */
void goto_eof                                (void);
void goto_bof                                (void);
void goto_eol                                (void);
void goto_bol                                (void);

/* For callbacks.c */
int byebye				     (void);
void capslock_toggle                         (GtkWidget *w, gpointer data);
void shift_on                                (GtkWidget *w, gpointer data);
/* void callback                                (GtkWidget *widget, 
   gpointer data); */

void status_report                           (GtkWidget *widget, 
					      gpointer data);
void status_logging_toggle                   (void);
void print_key                               (GtkWidget *widget, char *data);
void display_key                             (GtkWidget *widget, 
					      gpointer data);
void CutText                                 (GtkWidget *w, gpointer data);
void PasteText                               (GtkWidget *w, gpointer data);
void CopyText                                (GtkWidget *w, gpointer data);
void smack                                   (GtkWidget *w, gpointer data);
void save_output_text                        (GtkWidget *w, gpointer data);
void save_output_text_successful             (char *filename);
void save_output_text_error                  (char *error);
int delete_event                             (GtkWidget *foo, gpointer data);
void QuitApp                                 (gpointer callback_data,
					      guint callback_action,
					      GtkWidget *widget);
void Showmenu                                (gpointer callback_data,
					      guint callback_action,
					      GtkWidget *widget);
gint CloseApp                                (GtkWidget *widget, 
					      gpointer data);
void file_cancel_button                      (GtkWidget *w, 
					      GtkFileSelection *fs);

/* Prototypes for app.c */
void CLONE                                   (void);
GtkWidget *widget_from_xpm                   (GtkWidget *window, 
					      gchar **xpm_data);
int load_file                                (char *filename, 
					      GtkWidget *textbox);
void change_output                           (GtkWidget *w, int data);
void SMACK                                   (GtkWidget *foo, gpointer data);

void usage                                   (char *input);
void yes_no_popup_dialog                     (char *labelname, 
					      GtkSignalFunc yes, 
					      GtkSignalFunc no,
					      gpointer yesdata, 
					      gpointer nodata);
void clear_all_text_really                   (GtkWidget *widget, 
					      gpointer data);
void clear_all_text                          (GtkWidget *w, gpointer data);
void init_keyboard_stuff                     (char *input);
void About                                   (gpointer callback_data, 
					      guint callback_action, 
					      GtkWidget *widget);
void LICENSE                                 (gpointer callback_data, 
					      guint callback_action, 
					      GtkWidget *widget);
int DestroyDialog                            (GtkWidget *widget, 
					      gpointer *data);
void OpenFile                                (gpointer callback_data, 
					      guint callback_action,
					      GtkWidget *widget);
void NewFile                                 (gpointer callback_data, 
					      guint callback_action,
					      GtkWidget *widget);
void change_filename                         (GtkWidget *foo, gpointer data);
void CreateTable                             (GtkWidget *table, int columns, 
					      int rows, int buttons);
void file_ok_button                          (GtkWidget *w, 
					      GtkFileSelection *fs);
int get_index_of                             (char input);


/* For ui.c */
void setup_output_box                        (void);
void CreateMainWindow                        (void);
void select_all                              (GtkWidget *w, gpointer data);
void wrapper                                 (void);

/* For handlers.c */
/*
void error_handler                           (const gchar *buf);
void message_handler                         (const gchar *buf);
void print_handler                           (const gchar *buf);
void warning_handler                         (const gchar *buf);
*/

/* For html_mode.c */
/* More of these to come later */
/* Vanilla inserts text into output_text, and chocolate inserts into status */
void vanilla                                 (char *input);
void chocolate                               (char *input);

#ifdef INCLUDE_HTML   /* Only compile if defined */
void html_header                             (GtkWidget *w, int data);
void html_title                              (void);
void html_doctype                            (void);
void html_head                               (void);
void html_head_close                         (void);
void html_template                           (void);
void html_list_item                          (void);
void html_list                               (void);
void html_pre                                (void);
void html_paragraph                          (void);
void html_linebreak                          (void);
void html_image                              (void);
void html_center                             (void);
void html_link                               (void);
void html_html                               (void);
void html_italics                            (void);
void html_strong                             (void);
void html_bold                               (void);
void html_ftp                                (void);
void html_http                               (void);
void html_news                               (void);
void html_gopher                             (void);
void html_telnet                             (void);
void html_mailto                             (void);
void html_hr                                 (void);
void html_html_close                         (void);
void html_body                               (void);
void html_body_close                         (void);
void html_table                              (void);
void html_table_row                          (void);
void html_table_data                         (void);
void html_comment                            (void);
#endif /* INCLUDE_HTML */

/**************************** END OF PROTOTYPES ******************************/

/* GLOBAL VARIABLE UGLINESS */
GtkWidget *status;                  /* Status text widget at the bottom */
GtkWidget *output_text;             /* Main text window in the main app */

#ifdef APP_C
static GtkWidget *fileselection = NULL;
#ifndef CALLBACKS_C
static char *selectedfile = NULL;
#endif /* CALLBACKS_C */
#endif /* APP_C */

/*  static GtkWidget *table; */

/* Main program options - do not declare as static */
GTKeyboardOptions options;

/* This is the main menu at the top of the main window - it contains shortcuts,
 * HTML shortcuts, and just about everything that the app can do.
 */
#ifdef UI_C
static GtkItemFactoryEntry menu[]={
     {"/_File",                  NULL,          0,         0,  "<Branch>" },
     {"/_File/tearoff1",          NULL,          Showmenu,  0,  "<Tearoff>"},
     {"/_File/_New",              "<control>N",  NewFile,   0 },
     {"/_File/_Open",             "<control>O",  OpenFile,  0 },
     {"/_File/_Save",             "<control>S",  save_output_text, 0 },
     {"/_File/_Quit",             "<control>Q",  QuitApp,   0 },

     {"/_Edit",                   NULL,          0,         0,  "<Branch>" },
     {"/_Edit/tearoff1",          NULL,          Showmenu,  0,  "<Tearoff>"},
     {"/_Edit/_Cut",              "<control>X",  CutText,         0,  0},
     {"/_Edit/_Copy",             "<control>C",  CopyText,        0,  0},
     {"/_Edit/_Paste",            "<control>V",  PasteText,       0,  0},

#ifdef INCLUDE_HTML
     {"/_HTML",                  NULL,          0,         0,  "<Branch>" },
     {"/_HTML/tearoff1",          NULL,          Showmenu,  0,  "<Tearoff>"},
     {"/_HTML/Document",          NULL,          0,         0,  "<Branch>" },
     {"/_HTML/Document/Doctype",  NULL,          html_doctype, 0},
     {"/_HTML/Document/Open Doc", NULL,          html_html, 0 },
     {"/_HTML/Document/Close Doc",NULL,          html_html_close, 0 },
     {"/_HTML/Document/Head",     NULL,          html_head, 0 },
     {"/_HTML/Document/Close Head",NULL,         html_head_close, 0},
     {"/_HTML/Document/Title",    NULL,          html_title, 0 },
     {"/_HTML/Body",              NULL,          0,         0,  "<Branch>" },
     {"/_HTML/Body/Open Body",    NULL,          html_body, 0 },
     {"/_HTML/Body/Close Body",   NULL,          html_body_close, 0 },
     {"/_HTML/Text",              NULL,          0,       0,  "<Branch>" },
     {"/_HTML/Text/Bold",         NULL,          html_bold, 0},
     {"/_HTML/Text/Strong",       NULL,          html_strong, 0},
     {"/_HTML/Text/Italics",      NULL,          html_italics, 0},
     {"/_HTML/Text/Center",       NULL,          html_center, 0},
     {"/_HTML/Text/Preformatted", NULL,          html_pre,    0},
     {"/_HTML/Headers",           NULL,          0,         0,  "<Branch>" },
     {"/_HTML/Headers/H1",        NULL,          html_header, (int)H1 },
     {"/_HTML/Headers/H2",        NULL,          html_header, (int)H2 },
     {"/_HTML/Headers/H3",        NULL,          html_header, (int)H3 },
     {"/_HTML/Headers/H4",        NULL,          html_header, (int)H4 },
     {"/_HTML/Headers/H5",        NULL,          html_header, (int)H5 },
     {"/_HTML/Headers/H6",        NULL,          html_header, (int)H6 },
     {"/_HTML/Lists",             NULL,          0,          0,  "<Branch>" },
     {"/_HTML/Lists/New List",    NULL,          html_list,      0 },
     {"/_HTML/Lists/List Item",   NULL,          html_list_item, 0 },
     {"/_HTML/Tables",            NULL,          0,          0,  "<Branch>"},
     {"/_HTML/Tables/New Table",  NULL,          html_table,     0 },
     {"/_HTML/Tables/Table Row",  NULL,          html_table_row, 0 },
     {"/_HTML/Tables/Table Data", NULL,          html_table_data, 0},
     {"/_HTML/Protocols",         NULL,          0,          0,  "<Branch>"},
     {"/_HTML/Protocols/HTTP",    NULL,          html_http,  0},
     {"/_HTML/Protocols/FTP",     NULL,          html_ftp,   0},
     {"/_HTML/Protocols/News",    NULL,          html_news,  0},
     {"/_HTML/Protocols/Telnet",  NULL,          html_telnet,0},
     {"/_HTML/Protocols/Gopher",  NULL,          html_gopher,0},
     {"/_HTML/Paragraph",         NULL,          html_paragraph, 0},
     {"/_HTML/Image",             NULL,          html_image, 0},
     {"/_HTML/Link",              NULL,          html_link, 0 },
     {"/_HTML/Linebreak",         NULL,          html_linebreak, 0 },
     {"/_HTML/HR",                NULL,          html_hr, 0},
     {"/_HTML/mailto",            NULL,          html_mailto, 0},
     {"/_HTML/Comment",           NULL,          html_comment,    0},
     {"/_HTML/Full Template",     NULL,          html_template,   0},

#endif /* INCLUDE_HTML */

     /* SHORTCUTS MENU */
     {"/_Shortcuts",             NULL,          0,         0, "<Branch>"},
     {"/_Shortcuts/tearoff1",     NULL,          Showmenu,  0, "<Tearoff>"},
     {"/_Shortcuts/Quoting",      NULL,          0,         0, "<Branch>"},
     {"/_Shortcuts/Quoting/Quote",NULL,          SC_quote,            0   },
     {"/_Shortcuts/Quoting/Single Quote", NULL,  SC_single_quote,     0   },
     {"/_Shortcuts/Quoting/Book Quote",   NULL,  SC_book_quote,       0   },
     {"/_Shortcuts/Quoting/Back Quote",   NULL,  SC_backquote,        0   },
     {"/_Shortcuts/Quoting/Paragraph",    NULL,  SC_paragraph,        0   },
     {"/_Shortcuts/Quoting/Surround",     NULL,  SC_surround,         0   },
     {"/_Shortcuts/Group",        NULL,          0,         0, "<Branch>" },
     {"/_Shortcuts/Group/Parentheses",  NULL,    SC_parens,           0   },
     {"/_Shortcuts/Group/Angle Bracket", NULL,   SC_angle,            0   },
     {"/_Shortcuts/Group/Dbl Angle",    NULL,    SC_double_angle,     0   },
     {"/_Shortcuts/Group/Hi-Hat",       NULL,    SC_hihat,            0   },
     {"/_Shortcuts/Emoticons",    NULL,          0,         0, "<Branch>" },
     {"/_Shortcuts/Emoticons/Smiley",     NULL,  SC_smiley,           0   },
     {"/_Shortcuts/Emoticons/Winky",      NULL,  SC_winky,            0   },
     {"/_Shortcuts/Emoticons/Rose",       NULL,  SC_rose,             0   },
     {"/_Shortcuts/Pseudo-Underline", NULL,      SC_pseudo_underline, 0   },
     {"/_Shortcuts/Emphasize",    NULL,          SC_emphasize,        0   },

     /* OUTPUT MENU */
     {"/Output",                 NULL,          0,         0, "<Branch>"     },
     {"/Output/tearoff1",        NULL,          Showmenu,  0, "<Tearoff>"    },
     {"/Output/System Console",  NULL,          change_output, OUTPUT_CONSOLE},
     {"/Output/Printer",         NULL,          change_output, OUTPUT_PRINTER},
     {"/Output/Temp file",       NULL,         change_output, OUTPUT_TEMPFILE},

     /* MISC MENU */
     {"/_Misc",                  NULL,          0,         0, "<Branch>" },
     {"/_Misc/tearoff1",         NULL,          Showmenu,  0, "<Tearoff>"},
     {"/_Misc/Timestamp",         NULL,          SC_timestamp, 0 },
     {"/_Misc/Toggle Word Wrap",  NULL,          toggle_wordwrap, 0},
     {"/_Misc/Status Logging Toggle", NULL,      status_logging_toggle, 0 },
     {"/_Misc/Status Report",     NULL,          status_report,         0 },
     {"/_Misc/Beginning of line", NULL,          goto_bol,  0 },
     {"/_Misc/End of line",       NULL,          goto_eol,  0 },
     {"/_Misc/Beginning of file", NULL,          goto_bof,  0             },
     {"/_Misc/End of file",       NULL,          goto_eof,  0             },
     {"/_Misc/Left",              NULL,          cursor_left,0            },
     {"/_Misc/Right",             NULL,          cursor_right,0           },
     {"/_Misc/Clone window",      NULL,          CreateMainWindow, 0      },
     
     /* Help menu */
     {"/_Help",                  NULL,          0,         0, "<LastBranch>"},
     {"/_Help/tearoff1",          NULL,          Showmenu,  0,  "<Tearoff>"},
     {"/_Help/_About",            NULL,          About,  0 },
     {"/_Help/License",           NULL,          LICENSE, 0},
     {"/_Help/Splash",            NULL,          splash,  0},
}; /* End of menu (ItemFactory declaration) */
#endif /* UI_C */

/* Don't ask */
#ifdef UI_C
static int nmenu_items = sizeof(menu) / sizeof (menu[0]);
#endif

/* EOF */
