/* UI.C 
 * For use with GTKeyboard
 * by David Allen
 * s2mdalle@titan.vcu.edu http://opop.nols.com/
 * Released under the terms of the GNU General Public License
 *
 * This file contains some of the most important UI components of the program.
 * build_keyboard.c contains all of the code for building the keyboard.
 */
#define UI_C         /* File module */
#include "master.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <errno.h>
#include <signal.h>
#include <ctype.h>

#define BOXES 10
#define KEYS_PER_LINE 10

GtkWidget *window;

void CreateMainWindow(void)
{
     GtkWidget *box1;
     GtkWidget *mainbox, *h_mainbox;
     GtkWidget *KEYBOARD;
     GtkWidget *KEYPAD;
     GtkWidget *CURSORS;
     GtkWidget *FKEYS;
     GtkAccelGroup *accel_group;
     GtkItemFactory *item_factory;
     GtkWidget *subbox;
     GtkWidget *s_table, *vscroll;
     
     /* variables for the output section */
     GtkWidget *output_container, *output_subcontainer, *output_table;
     GtkWidget *output_vscroll, *output_label, *output_button;
     GtkWidget *output_savebutton, *output_aboutbutton, *output_licensebutton;
     GtkWidget *output_clearbutton, *output_clearstatusbutton,
	  *output_openbutton, *output_sa;
     GtkWidget *statusreport;

     int x, boxcounter=0;           /* Boxcounter is an index for miscbox */

     /* This needs to be set up before the buttons get constructed */
     output_text = gtk_text_new(NULL, NULL);

     window = gtk_window_new(GTK_WINDOW_TOPLEVEL);

     /* Window size - defined in the header */
     /* If you take it out, it should still shrink to the right size.
      * Maybe.  */
     /* gtk_widget_set_usize(window, WIDTH, HEIGHT);  */

     /* Set up s_table, which contains a scrollbar and the status widget 
      * that tells the user what's going on with the program 
      */
     s_table = gtk_table_new(2, 2, FALSE);
     gtk_widget_show(s_table);

     status = gtk_text_new(NULL, NULL);
     gtk_widget_set_usize(status, (WIDTH-20), STATUS_HEIGHT);
     gtk_text_set_editable(GTK_TEXT(status), FALSE);
     gtk_text_set_point(GTK_TEXT(status), 0);

     gtk_table_attach(GTK_TABLE(s_table), status, 0, 1, 0, 1,
		      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(status);

     vscroll = gtk_vscrollbar_new(GTK_TEXT(status)->vadj);
     gtk_table_attach(GTK_TABLE(s_table), vscroll, 1, 2, 0, 1, 
		      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(vscroll);

     /* Set up app exiting signals */
     gtk_signal_connect(GTK_OBJECT(window), "delete_event",
			GTK_SIGNAL_FUNC(CloseApp), NULL);

     /* Start creating the file menus, etc. */
     accel_group = gtk_accel_group_new();
     item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR,
					 "<blah>",
					 accel_group);
     gtk_item_factory_create_items(item_factory, nmenu_items, menu,
				   NULL);
     gtk_accel_group_attach(accel_group, GTK_OBJECT(window));

     /* Set title of app */
     gtk_window_set_title(GTK_WINDOW(window), APPNAME);

     gtk_container_border_width(GTK_CONTAINER(window), 0);
     
     box1 = gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(window), box1);
     
     mainbox   = gtk_vbox_new(FALSE, 0);
     h_mainbox = gtk_hbox_new(FALSE, 20);
     subbox    = gtk_hbox_new(FALSE, 0);

     KEYBOARD = build_keyboard(status);
     CURSORS  = build_cursorpad(window);
     FKEYS    = build_fkeys();
     KEYPAD   = build_keypad(status);

     /* Pack the ENTIRE keyboard into the widget mainbox */

     gtk_box_pack_start(GTK_BOX(mainbox), FKEYS, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(h_mainbox), KEYBOARD, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(h_mainbox), CURSORS, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(h_mainbox), KEYPAD,   FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(mainbox), h_mainbox, FALSE, FALSE, 0);

     gtk_widget_show(FKEYS);
     gtk_widget_show(h_mainbox);
     gtk_widget_show(KEYBOARD);
     gtk_widget_show(CURSORS);
     gtk_widget_show(KEYPAD);

     gtk_box_pack_start(GTK_BOX(mainbox), s_table, FALSE, FALSE, 0);

     x=0;
     boxcounter=0;

/* Create the widget, read only, point to 0, etc. */

     /* Set up the widget with default crap */
     chocolate(STATUS_INTRO);
     chocolate(SHIFT_IS_OFF);
     chocolate(CAPS_LOCK_IS_OFF);
     
     /* Turn status logging on by default.  It starts as zero, this will turn
      * it to 1, and print the status of the logging itself to the screen
      */
     status_logging_toggle(); 

     /* Now pack s_table, which includes status with the slidebar */
     gtk_box_pack_start(GTK_BOX(box1),
			gtk_item_factory_get_widget(item_factory, "<blah>"),
			FALSE, FALSE, 0);
     
     /* Pack the main box in */
     gtk_box_pack_start(GTK_BOX(box1),
			mainbox, FALSE, FALSE, 0);

/****************************************************************************/
     /* BEGIN OUTPUT TEXT / FOOTER SECTION */
     
     output_container = gtk_vbox_new(FALSE, 0);
     output_subcontainer=gtk_hbox_new(FALSE, 0);
     output_table =  gtk_table_new(2, 2, FALSE);

     gtk_widget_set_usize(output_text, OUTPUT_WIDTH, OUTPUT_HEIGHT);
     gtk_text_set_editable(GTK_TEXT(output_text), TRUE);
     gtk_text_set_point(GTK_TEXT(output_text), 0);

     gtk_table_attach(GTK_TABLE(output_table), output_text, 0, 1, 0, 1,
		      GTK_EXPAND | GTK_SHRINK | GTK_FILL,
		      GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(output_text);

     gtk_widget_show(output_table);

     output_vscroll = gtk_vscrollbar_new(GTK_TEXT(output_text)->vadj);
     gtk_table_attach(GTK_TABLE(output_table), output_vscroll, 1, 2, 0, 1, 
		      GTK_FILL, GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
     gtk_widget_show(output_vscroll);

     output_label = gtk_label_new("OUTPUT:");

     gtk_box_pack_start(GTK_BOX(output_container), output_label, 
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_container), output_table, 
			FALSE, FALSE, 0);

     gtk_widget_show(output_container);
     
     /* New buttons galore */
     output_sa = gtk_button_new_with_label(" Select All ");
     output_button = gtk_button_new_with_label("  Close App  ");
     output_savebutton = gtk_button_new_with_label("  Save File  ");
     output_aboutbutton = gtk_button_new_with_label(" About ");
     output_licensebutton = gtk_button_new_with_label(" License ");
     output_clearbutton = gtk_button_new_with_label(" Clear All ");
     output_clearstatusbutton = gtk_button_new_with_label(" Clear Status ");
     output_openbutton = gtk_button_new_with_label("  Open File  ");
     statusreport      = gtk_button_new_with_label(" Status ");

     /* Show a bunch of widgets */
     gtk_widget_show(output_button);
     gtk_widget_show(output_sa);
     gtk_widget_show(output_savebutton);
     gtk_widget_show(output_aboutbutton);
     gtk_widget_show(output_licensebutton);
     gtk_widget_show(output_clearbutton);
     gtk_widget_show(output_clearstatusbutton);
     gtk_widget_show(output_openbutton);
     gtk_widget_show(statusreport);

     /* Pack all of the base row buttons into subcontainer in the following
      * order:  =SAVE=OPEN=CLEAR=ABOUT=QUIT=
      */
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_savebutton, 
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_openbutton,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_sa,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_clearbutton,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_clearstatusbutton,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_aboutbutton, 
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_licensebutton,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), statusreport,
			FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(output_subcontainer), output_button, 
			FALSE, FALSE, 0);

     /* Now pack the smaller container into the larger */
     gtk_box_pack_start(GTK_BOX(output_container), output_subcontainer, 
			FALSE, FALSE, 0);

     /* Signal Mapping for bottom row buttons */
     
     /* Quit button mapped to CloseApp - gtk_main_quit and so on. */
     gtk_signal_connect(GTK_OBJECT(output_sa), "clicked",
			GTK_SIGNAL_FUNC(select_all), output_text);
     gtk_signal_connect(GTK_OBJECT(statusreport), "clicked",
			GTK_SIGNAL_FUNC(status_report), status);
     gtk_signal_connect(GTK_OBJECT(output_button), "clicked", 
			GTK_SIGNAL_FUNC(CloseApp), NULL);
     /* Save button mapped to save_output_text, text widget as arg */
     gtk_signal_connect(GTK_OBJECT(output_savebutton), "clicked", 
			GTK_SIGNAL_FUNC(save_output_text), output_text);
     /* About button mapped to About, no args */
     gtk_signal_connect(GTK_OBJECT(output_aboutbutton), "clicked", 
			GTK_SIGNAL_FUNC(About), NULL);
     /* Open button mapped to OpenFile, no args */
     gtk_signal_connect(GTK_OBJECT(output_openbutton), "clicked",
			GTK_SIGNAL_FUNC(OpenFile), NULL);
     /* License button mapped to LICENSE, no args */
     gtk_signal_connect(GTK_OBJECT(output_licensebutton), "clicked",
			GTK_SIGNAL_FUNC(LICENSE), NULL);
     /* Clear button mapped to output_clearbutton, (pop up dialog) */
     gtk_signal_connect(GTK_OBJECT(output_clearbutton), "clicked",
			GTK_SIGNAL_FUNC(clear_all_text), output_text);
     gtk_signal_connect(GTK_OBJECT(output_clearstatusbutton), "clicked",
			GTK_SIGNAL_FUNC(clear_all_text), status);

     /* Pack the main box into the main window, and we're ready. */
     gtk_box_pack_start(GTK_BOX(box1), output_container, FALSE, FALSE, 0);

     /* END OUTPUT TEXT / FOOTER SECTION */
/*****************************************************************************/

     gtk_widget_show_all(window);
     splash();
} /* End CreateMainWindow */
/* END MAIN WINDOW CONSTRUCTION FUNCTION CREATEMAINWINDOW */

void wrapper(void)
{
     yes_no_popup_dialog("Are you sure you want to quit?",
			 GTK_SIGNAL_FUNC(CloseApp), NULL, NULL, NULL);
}

void select_all(GtkWidget *w, gpointer data)
{
     GtkWidget *foo = (GtkWidget *)data;
     gtk_editable_select_region(GTK_EDITABLE(foo), (gint)0,
				gtk_text_get_point(GTK_TEXT(foo)));
}


/* EOF */






